/*  gtkcutlass-cutlass.c - this is the main goo for gluing cutlass to
*  gtkcutlass (and, hopefully, making it work)
*/


// First stab - Finland, 8/14/04

#include <gtkcutlass.h>
#include <string.h>		/* for memcpy */

// local use only
gint setup_textchat_window ( conv_conn *cc );
gboolean parse_gtkc_messages ( gpointer data );
gint already_in_list ( GSList *list, struct cut_action_obj *action_obj );

gint gtkcutlass_conn_finish_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    conv_conn *cc;  // a pointer to the associated conversation, if there is one.
    extern	gui_status *currentstatus;
    gtkc_message_q *msg;

    // if the message has already been put on the queue, exit.
    if ( already_in_list( currentstatus->messages, action_object ) )
        return ( 0 );

    cc = setup_incoming_conversation ( handle, action_object );
    if ( cc == NULL )
    {
        g_warning ( " Couldn't set up new connection! " );
        return ( -1 );
    }

    msg = message_init();
    msg->cc = cc;

    msg->messagetype = NEW_TEXTWINDOW;
    msg->action_object = action_object;
    msg->id = ( guint ) time( NULL );
    msg->read = FALSE;
    // append to the global list
    currentstatus->messages = g_slist_append ( currentstatus->messages, msg );
    currentstatus->conversations = g_slist_append ( currentstatus->conversations, cc );
    return ( 0 );
}

gint gtkcutlass_conn_new_incoming_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    /* this is never called.  <sigh> */
    g_print ( "gtkcutlass_conn_new_incoming_handler: called\n" );
    return ( 0 );  // 1 means not accepted, 0, is accepted
}

gint gtkcutlass_drop_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    conv_conn *cc;
    uint8_t fp[ CUT_ID_LEN ];

    /* need to check to see if there is an existing window for this conversation.  If so,
     * notify the user, then kill it.
     */

    memcpy ( fp, cutlass_action_fingerprint ( action_object ), CUT_ID_LEN );
    cc = find_conv_conn_from_connection( fp );

    conv_conn_destroy( cc );
    g_print ( "drop handler called\n" );
    return ( 0 );
}

gint gtkcutlass_message_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    conv_conn *cc;
    uint8_t fp[ CUT_ID_LEN ];
    gtkc_message_q *msg;
    extern gui_status *currentstatus;

    memcpy ( fp, cutlass_action_fingerprint ( action_object ), CUT_ID_LEN );

    cc = find_conv_conn_from_connection ( fp );
    if ( cc == NULL )
    {
        g_warning ( "No conversation found. Bad state.\n" );
        exit ( 1 );
    }

    // need to add message to queue for GUI updates, here.
    msg = message_init();
    msg->cc = cc;
    msg->messagetype = TEXTMESSAGE;
    msg->cc->name = cutlass_action_rnick( action_object );
    msg->data = cutlass_action_msg( action_object );
    msg->action_object = action_object;
    msg->id = ( guint ) time( NULL );
    msg->read = FALSE;
    currentstatus->messages = g_slist_append ( currentstatus->messages, msg );
    return ( 0 );
}

gint gtkcutlass_timeout_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_timeout_handler: called\n" );
    /* not entirely sure when this is called. or what to do with it. */
    return ( 0 );
}

gint gtkcutlass_group_join_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_group_join_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_group_join_req_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_group_join_req_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_group_invite_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_group_invite_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_group_leave_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_group_leave_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_file_offer_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    /* this is called when the user is offered a file */
    extern gui_status *currentstatus;
    conv_conn *cc;
    uint8_t fp[ CUT_ID_LEN ];
    gtkc_message_q *msg = NULL;

    memcpy ( fp, cutlass_action_fingerprint ( action_object ), CUT_ID_LEN );
    cc = find_conv_conn_from_connection( fp );
    if ( cc == NULL )
        g_warning ( "unknown connection attempting to ask for to send a file" );
    msg = message_init();
    msg->cc = cc;
    msg->action_object = action_object;
    msg->messagetype = NEW_FILEWINDOW;
    msg->id = ( guint ) time ( NULL );
    msg->read = FALSE;
    currentstatus->messages = g_slist_append ( currentstatus->messages, msg );
    return ( 0 );
}
gint gtkcutlass_file_accept_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_file_accept_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_file_recv_done_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    //	g_print("gtkcutlass_file_recv_done_handler: called\n");
    return ( 0 );
}
gint gtkcutlass_file_send_done_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_file_send_done_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_sound_offer_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    // this is called when the user needs to be asked to accept an incoming call.
    // Since I can't do GUI stuff here (outside the gtk_main loop) I will stuff a message on the queue to be parsed later (within gtk_main)
    extern gui_status *currentstatus;
    conv_conn *cc;
    uint8_t fp[ CUT_ID_LEN ];
    gtkc_message_q *msg = NULL;

    memcpy ( fp, cutlass_action_fingerprint ( action_object ), CUT_ID_LEN );
    cc = find_conv_conn_from_connection( fp );
    if ( cc == NULL )
        g_warning ( "unknown connection attempting to ask for audio (?odd?)" );
    msg = message_init();
    msg->cc = cc;
    msg->action_object = action_object;
    msg->messagetype = AUDIOREQUEST;
    msg->id = ( guint ) time ( NULL );
    msg->read = FALSE;
    currentstatus->messages = g_slist_append ( currentstatus->messages, msg );
    return ( 0 );
}
gint gtkcutlass_sound_accept_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_sound_accept_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_sound_done_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    g_print( "gtkcutlass_sound_done_handler: called\n" );
    return ( 0 );
}
gint gtkcutlass_system_message_handler ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    gchar *message;

    message = cutlass_action_msg ( action_object );
    g_print ( "CSM: %s", message );

    g_free ( message );
    return ( 0 );
}


gint setup_textchat_window ( conv_conn *cc )
{
    /* this functiuon should also destroy the window in the event that
     * it has some kinda failure, and has to return early.
     */
    extern gui_status * currentstatus;
    GladeXML *windowxml;
    GtkWindow *window;
    gchar *name;
    GtkEntry *newentry;
    GtkTextView *newview;


    gdk_threads_enter();

    windowxml = ( GladeXML * ) glade_xml_new_from_buffer ( currentstatus->XMLstuff, currentstatus->XMLstuffsize, "TextConversationWindow", NULL );
    if ( windowxml == NULL )
    {
        gdk_threads_leave();
        return ( -1 );
    }
    glade_xml_signal_autoconnect ( windowxml );
    window = ( GtkWindow * ) glade_xml_get_widget ( windowxml, "TextConversationWindow" );
    if ( window == NULL )
    {
        gdk_threads_leave();
        return ( -2 );
    }
    name = g_strdup_printf( "Text Conversation with %s", cc->name );
    gtk_window_set_title( GTK_WINDOW( window ), name );
    g_free ( name );
    newentry = ( GtkEntry * ) glade_xml_get_widget ( windowxml, "entry1" );
    if ( newentry == NULL )
    {
        gdk_threads_leave();
        return ( -3 );
    }
    newview = ( GtkTextView * ) glade_xml_get_widget ( windowxml, "textconversation_textview" );
    if ( newview == NULL )
    {
        gdk_threads_leave();
        return ( -4 );
    }
    cc->textview = newview;
    cc->entry = newentry;
    cc->winder = gtk_widget_get_toplevel( GTK_WIDGET( newview ) );
    gdk_threads_leave();
    while ( gtk_events_pending() )
        gtk_main_iteration();
    return ( 0 );
}


gint already_in_list ( GSList *list, struct cut_action_obj *action_obj )
{
    GSList *ltmp = NULL;

    if ( ( list == NULL ) || ( action_obj == NULL ) )
        return ( 0 );
    if ( list->data == NULL )
        return ( 0 );
    ltmp = list;
    while ( ltmp != NULL )
    {
        if ( ltmp->data == NULL )
            return ( 0 );
        if ( ltmp->data == action_obj )
            return ( 1 );
        ltmp = ltmp->next;
    }
    return ( 1 );
}

