/*
gtkcutlass-newcontact.c - the callbacks for the new contact window
(c) 2004 - John Schweitzer
*/



#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <glade/glade.h>

#include <gtkcutlass.h>
#include <gtkcutlass-utilities.h>

gint inuse=0;
gint changed = 0;
gchar *name = NULL;
gchar *windowtitle = NULL;
gint namelen = 0;
gchar *hostname = NULL;
gint hostnamelen = 0;
guint preferedport = CUT_DEFAULT_PORT;
gchar *nickname = NULL;
extern user_info *userstuff;

void on_NewContactPeepNameEntry_changed ( GtkWidget *widget, gpointer user_data )
{
    gint length = 0;
    changed = 1;

    // what is the length of the entered string?

    length = strlen( gtk_entry_get_text( ( GtkEntry * ) widget ) );

    if ( length >= 64 )
        return ;

    if ( name != NULL )
        g_free( name );
    name = ( gchar * ) g_strndup( gtk_entry_get_text( ( GtkEntry * ) widget ), 63 );
    windowtitle = g_strdup_printf( "New Contact - %s", name );

    gtk_window_set_title( GTK_WINDOW( gtk_widget_get_toplevel( widget ) ), windowtitle );

    //g_print("NCPNE - changed :%s\n",name);
};

void on_NewContactCancelButton_activate ( GtkWidget *widget, gpointer user_data )
{
    if ( changed != 0 )
        if ( YesNo( widget, "Entry Changed.  Really Cancel?" ) == GTK_RESPONSE_NO )
            return ;
    gtk_widget_destroy( gtk_widget_get_toplevel( widget ) );
	inuse = 0;
};


void on_NewContactOKButton_activate ( GtkWidget *widget, gpointer user_data )
{
    contact_info * newcontact;
    gint validation = 0;

    if ( changed == 1 )
    {
        newcontact = contact_info_init();
        newcontact->contactname = g_strndup( name, 64 );
        newcontact->contacthostname = g_strndup( hostname, 128 );
        newcontact->nickname = g_strndup( nickname, 64 );
        newcontact->port = preferedport;

        validation = validate_contactinfo ( newcontact );
        if ( validation == 0 )
        {
            if ( check_contact_exists( newcontact, userstuff ) != 0 )
            {
                tellem ( "Duplicate contact\n" );
                return ;
            }
            userstuff->contacts = g_slist_append( userstuff->contacts, newcontact );
            gtk_widget_destroy( gtk_widget_get_toplevel( widget ) );
            append_treestore_contactinfo( newcontact );
			save_contact_list();
			inuse=0;
        }
        else
            switch ( validation )
            {
                case GTKC_ERR_NONAME:
                    {
                        tellem( "Contact needs a name" );
                        break;
                    }
                case GTKC_ERR_NULL:
                    {
                        tellem( "I was handed a NULL pointer.  Notify the developers." );
                        break;
                    }
                default:
                    {
                        tellem( "There is something so wrong with this contact that there is no error code for it.  (!)" );
                        break;
                    }
            }


    return ;

    }
};

void on_NewContactHostnameEntry_changed ( GtkWidget *widget, gpointer user_data )
{
    hostname = ( gchar * ) g_strndup( gtk_entry_get_text( ( GtkEntry * ) widget ), GTKC_MAX_HOSTNAME_LEN );
    changed = 1;
};

void on_NewContactWindow_destroy_event ( GtkWidget *widget, gpointer user_data )
{
    if ( changed != 0 )
        if ( YesNo( widget, "Entry Changed.  Really Cancel?" ) ==
                GTK_RESPONSE_NO )
            return ;
    gtk_widget_destroy ( widget );
	inuse = 0;
};

void on_NewContactWindow_delete_event ( GtkWidget *widget, gpointer user_data )
{
    on_NewContactWindow_destroy_event ( widget, user_data );
};

void on_Nicknameentry_changed ( GtkWidget *widget, gpointer user_data )
{
    gint length = 0;
    changed = 1;

    length = strlen( gtk_entry_get_text( ( GtkEntry * ) widget ) );
    if ( length >= 64 )
        return ;
    if ( name != NULL )
        g_free( nickname );
    nickname = ( gchar * ) g_strndup( gtk_entry_get_text( ( GtkEntry * ) widget ), 63 );
};

void on_newcontactport_changed ( GtkWidget *widget, gpointer user_data )
{
    gdouble tempval = CUT_DEFAULT_PORT;

    tempval = gtk_spin_button_get_value( GTK_SPIN_BUTTON( widget ) );

    changed = 1;
    if ( ( tempval <= 65535 ) && ( tempval >= 0 ) )
        preferedport = ( guint ) tempval;
}

gint check_contact_exists( contact_info *ci, user_info *ui )
{
    // FIXME: The comparison is busted for UTF8
    GSList * tmp = NULL;
    gint match = 0;
    contact_info *meh;

    if ( ci == NULL )
        return -1;
    if ( ui == NULL )
        return -1;

    tmp = ui->contacts;
    while ( tmp != NULL )
    {
        meh = ( contact_info * ) tmp->data;
        if ( g_ascii_strcasecmp( ci->contactname, meh->contactname ) == 0 )
            match += 1;
        if ( g_ascii_strcasecmp( ci->nickname, meh->nickname ) == 0 )
            match += 1;
        tmp = tmp->next;
    }
    return ( match );
}

void run_newcontact_window ( GtkWidget *widget, gpointer user_data )
{
	GladeXML * NewContactWindowXML;
	extern gui_status *currentstatus;

	if ( inuse != 0 )
	{
		send_statusbar_message ("Meh","newcontactwindow already open");
		return;
	}
	inuse = 1;
	NewContactWindowXML = glade_xml_new_from_buffer( currentstatus->XMLstuff, currentstatus->XMLstuffsize, "NewContactWindow", NULL ); 
	if ( NewContactWindowXML == NULL )
	{
		g_error( "Couldn't create window for new contact.  (gtkcutlass.glade not found?) Aborting.\n" );
		exit( -1 );
	}
	glade_xml_signal_autoconnect( NewContactWindowXML );
};
