/* gtkcutlass-textconversation.c - connect to, and have a conversation with,
* the currently selected user.
*/

// 18 August 2004 - On the train from Helsinki to Turku. :)

#include <gtkcutlass.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>


void TextConnect( void )
{
    extern user_info * userstuff;
    extern gui_status *currentstatus;
    contact_info *contact = NULL;
    conv_conn *newconv_conn;

    if ( ( userstuff == NULL ) || ( currentstatus == NULL ) )
    {
        g_print( "something is null\n" );
        gdk_threads_leave();
        return ;
    }

    contact = get_contact_from_name ( get_treeview_selection_name() );
    if ( contact == NULL )
    {
        tellem ( "Invalid contact" );
        return ;
    }
    currentstatus->something_selected = TRUE;

    if ( ( newconv_conn = get_conversation_from_name ( get_treeview_selection_name() ) ) == NULL )
        if ( connect_to_contact( contact ) != 0 )
        {
            tellem ( "Error starting connection" );
            g_warning ( "couldn't connect -- !!\n" );
            return ;
        }
    return ;
}


void on_textconversation_disconnect_button_clicked ( GtkWidget *widget, gpointer user_data )
{
    extern user_info * userstuff;
    conv_conn *cc = NULL;

    cc = find_conv_conn_from_toplevel( gtk_widget_get_toplevel( widget ) );
    if ( cc == NULL )
        return ;

    if ( YesNo ( gtk_widget_get_toplevel( widget ), "Really disconnect user?" ) == GTK_RESPONSE_NO )
        return ;

    if ( cutlass_conn_shutdown ( userstuff->cutlass_info, cc->connection ) < 0 )
        g_warning ( "Error disconnecting -- " );
    cc->is_connected = 0;
    gtk_object_destroy ( GTK_OBJECT ( cc->winder));
    conv_conn_destroy( cc );
}

void on_entry1_delete_text ( GtkWidget *widget, gpointer user_data )
{
    // g_print ("entry1: Text deleted\n");
}

void on_textconversation_afk_button_clicked ( GtkWidget *widget, gpointer user_data )
{
    NotImplemented( gtk_widget_get_toplevel( widget ) );
}

void on_entry1_changed ( GtkWidget *widget, gpointer user_data )
{
    //g_print ("entry1: Text changed\n");
}

void on_textconversation_startaudio_button_clicked ( GtkWidget *widget, gpointer user_data )
{
    extern user_info *userstuff;
    conv_conn *cc = NULL;
    gint x = 0;

    send_statusbar_message( "meh", "audio connection requested" );
    cc = find_conv_conn_from_toplevel ( gtk_widget_get_toplevel( widget ) );
    if ( cc == NULL )
        return ;

    x = cutlass_audio_connect ( userstuff->cutlass_info, cc->connection );
    if ( x != 0 )
        g_warning ( "audio connect failed." );
    return ;
}

void on_textconversation_stopaudio_button_clicked ( GtkWidget *widget, gpointer user_data )
{
    extern user_info * userstuff;
    conv_conn *cc = NULL;

    cc = find_conv_conn_from_toplevel ( gtk_widget_get_toplevel ( widget ) );
    if ( cc == NULL )
        return ;
    if ( cutlass_audio_disconnect ( userstuff->cutlass_info, cc->connection ) != 0 )
        g_warning ( "couldn't shut down audio connection" );
    return ;
}

gint append_message_to_textview( GtkTextView *widget, gchar *message )
{
    GtkTextBuffer * theoutputbuffer;
    GtkTextIter iter;


    if ( message == NULL )
        return ( -1 );

    if ( widget == NULL )
        return ( -1 );
    gdk_threads_enter();
    theoutputbuffer = gtk_text_view_get_buffer ( widget );
    if ( theoutputbuffer == NULL )
    {
        gdk_threads_leave();
        g_warning ( "Cannot find a textbuffer to work with. Argh.\n" );
        return ( -1 );
    }

    // grab an 'iter' at the end of the buffer
    gtk_text_buffer_get_end_iter ( GTK_TEXT_BUFFER( theoutputbuffer ), &iter );

    // add the message at the iter
    gtk_text_buffer_insert( theoutputbuffer, &iter, message, -1 );
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(widget), &iter, 0.0, TRUE, 0.0, 1.0);

    while ( gtk_events_pending() )
        gtk_main_iteration();

    gdk_threads_leave();
    return ( 0 );
}

void on_textconversation_entry1_activate ( GtkWidget *widget, gpointer user_data )
{
    gchar * tmpbuffer;
    gchar * buffer_to_cutlass;
    extern user_info *userstuff;
    conv_conn	*currentconversation;
    gint	x;

    gdk_threads_enter();

    // get the text, first.
    tmpbuffer = gtk_editable_get_chars ( ( GtkEditable * ) widget, 0, -1 );
    if ( tmpbuffer == NULL )
    {
        g_warning ( "Strange.  empty line.  Ignoring\n" );
        gdk_threads_leave();
        return ;
    }
    buffer_to_cutlass = g_strdup_printf( "%s\n", tmpbuffer );

    // add the '/n' back.
    tmpbuffer = g_strdup_printf( "%s> %s\n", userstuff->defaultalias, tmpbuffer );
    currentconversation = find_conv_conn_from_textentry( ( GtkEntry * ) widget );
    if ( currentconversation == NULL )
    {
        g_warning( "couldn't find conversation (!!!)\n" );
        gdk_threads_leave();
        return ;
    }

    if ( ( currentconversation->was_connected = 0 ) && ( currentconversation->is_connected == 1 ) )
    {
        tellem( "Conversation has ended." );
        // should kill the connection (via libcutlass) here.
        gdk_threads_leave();
        return ;
    }

    if ( currentconversation->is_connected == 1 )
    {
        tellem( "Cannot send message, not connected" );
        gdk_threads_leave();
        return ;
    }

    // append to msgview, then send the same message via cutlass.
    if ( append_message_to_textview( currentconversation->textview, tmpbuffer ) != 0 )
    {
        g_warning( "Error appending new message" );
        gdk_threads_leave();
        return ;
    }

    if ( currentconversation->connection == NULL )
    {
        tellem ( "cannot find connection (?!)" );
        gdk_threads_leave();
        return ;
    }

    x = cutlass_msg_send( userstuff->cutlass_info, currentconversation->connection, buffer_to_cutlass );
    if ( -1 == x )
    {
        if ( YesNo ( widget,
                     "There was an error sending the last message to the contact. Do you wish to keep the conversation open?" ) == GTK_RESPONSE_NO )
            // FIXME: <sigh> we need to *attempt* to shut down the conversation cleanly.
            gtk_object_destroy ( GTK_OBJECT ( gtk_widget_get_toplevel( GTK_WIDGET( widget ) ) ) );

        gdk_threads_leave();
        g_free ( tmpbuffer );
        g_free ( buffer_to_cutlass );
        return ;
    }

    gtk_editable_delete_text( ( GtkEditable * ) widget, 0, -1 );

    while ( gtk_events_pending() )
        gtk_main_iteration();

    gdk_threads_leave();
    g_free ( buffer_to_cutlass );
    g_free( tmpbuffer );
    return ;
}

void IncomingTextConnect( cutlass_t *handle, struct cut_action_obj *action_object, conv_conn *cc )
{
    extern user_info * userstuff;
    extern gui_status *currentstatus;
    GtkWindow *window_object = NULL;
    GtkEntry *newentry = NULL;
    GtkTextView *newview = NULL;
    GladeXML *Window;
    GtkTextBuffer *theoutputbuffer;
    GtkTextIter iter;
    gchar *contactname = NULL;
    gchar *message = NULL;

    gdk_threads_enter();

    if ( ( userstuff == NULL ) || ( currentstatus == NULL ) )
    {
        g_print( "something is null\n" );
        gdk_threads_leave();
        return ;
    }

    currentstatus->activeconversation = TRUE;

    Window = ( GladeXML * ) glade_xml_new (GTK_CUTLASS_GLADEDIR "gtkcutlass.glade", "TextConversationWindow", NULL );

    if ( Window == NULL )
    {
        tellem ( " Cannot create text conversation window.  This is somewhat bad.  Bailing." );
        gdk_threads_leave();
        g_error( " Cannot load TextConversationWindow from gtkcutlass.glade " );
    }

    window_object = ( GtkWindow * ) glade_xml_get_widget ( Window, "TextConversationWindow" );
    contactname = cutlass_action_rnick ( action_object );
    if ( contactname != NULL )
    {
        gtk_window_set_title( GTK_WINDOW( window_object ), g_strdup_printf( "Text Conversation with %s", contactname ) );
        cc->name = g_strdup ( contactname );
    }
    else
    {
        gtk_window_set_title( GTK_WINDOW( window_object ), g_strdup_printf( "Text Conversation with %s", "J Random User" ) );
        cc->name = g_strdup ( "Unknown" );
    }

    newentry = ( GtkEntry * ) glade_xml_get_widget( Window, "entry1" );
    if ( newentry == NULL )
    {
        g_warning ( "couldn't find 'entry1'\n" );
        gdk_threads_leave();
        gtk_object_destroy ( ( GtkObject * ) Window );
        g_free ( contactname );
        return ;
    }

    newview = ( GtkTextView * ) glade_xml_get_widget( Window, "textconversation_textview" );
    if ( newview == NULL )
    {
        g_warning( "couldn't find 'textconversation_textview'\n" );
        gtk_object_destroy ( ( GtkObject * ) Window );
        gdk_threads_leave();
        g_free ( contactname );
        return ;
    }

    glade_xml_signal_autoconnect ( Window );

    // make the entry
    if ( newview == NULL )
        g_print ( "newview is null\n" );
    cc->textview = newview;
    if ( newentry == NULL )
        g_print ( "newentry is null\n" );
    cc->entry = newentry;

    // add the connecting message
    theoutputbuffer = gtk_text_view_get_buffer ( newview );
    gtk_text_buffer_get_end_iter ( GTK_TEXT_BUFFER( theoutputbuffer ), &iter );
    message = g_strdup_printf( "[ Connected to %s@%s:%u ]\n", cc->name, cc->addr, cc->port );
    gtk_text_buffer_insert( theoutputbuffer, &iter, message, -1 );

    currentstatus->activeconversation = TRUE;

    while ( gtk_events_pending() )
        gtk_main_iteration();

    gdk_threads_leave();
    g_free( message );
}

void on_textconversation_request_send_file_button_clicked ( GtkWidget *widget, gpointer user_data )
{
    // the user has clicked on the 'send file' button
    gchar * filename = NULL;
    GtkWidget *fileselection;
    extern	user_info *userstuff;
    conv_conn *cc;

    if ( widget == NULL )
    {
        g_warning ( "NULL widget passed by callback (!?)" );
        return ;
    }

    cc = find_conv_conn_from_toplevel ( gtk_widget_get_toplevel ( widget ) );
    if ( cc == NULL )
        return ;  // there is no conversation for this window.

    fileselection = gtk_file_chooser_dialog_new ( "Select File to Send", GTK_WINDOW ( gtk_widget_get_toplevel ( widget ) ),
                    GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                    GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL );
    if ( gtk_dialog_run ( GTK_DIALOG( fileselection ) ) == GTK_RESPONSE_CANCEL )
        return ;
    filename = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER( fileselection ) );

    // FIXME: I should record these.
    if ( cutlass_file_offer ( userstuff->cutlass_info, cc->connection, filename ) == -1 )
        tellem ( "Error sending file" );

    g_free( filename );
    gtk_widget_destroy( fileselection );
}
