/*
gtkcutlass-utilities.c - Some misc utilities 
(c) 2004 - John Schweitzer
*/



#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <pwd.h>
#include <fcntl.h>
#include <errno.h>
#include <time.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <libgen.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <glade/glade.h>

/* #include <cutlass.h> */
#include <gtkcutlass.h>

gboolean parse_gtkc_messages ( gpointer data );
gint setup_textchat_window ( conv_conn *cc );

void NotImplemented ( GtkWidget *window )
{

    GtkWidget * tempwidget;
    tempwidget = gtk_message_dialog_new
                 ( NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                   "Not Implemented" );
    gtk_dialog_run ( GTK_DIALOG( tempwidget ) );
    gtk_widget_destroy( tempwidget );
};

gint YesNo ( GtkWidget *window, gchar *message )
{
    gint result = 0;

    GtkWidget *tempwidget;
    tempwidget = gtk_message_dialog_new
                 ( NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
                   message );
    result = gtk_dialog_run ( GTK_DIALOG( tempwidget ) );
    gtk_widget_destroy( tempwidget );
    return ( result );
};
void tellem ( gchar *message )
{
    GtkWidget * tempwidget;
    tempwidget = gtk_message_dialog_new
                 ( NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
                   GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, message );
    gtk_dialog_run ( GTK_DIALOG( tempwidget ) );
    gtk_widget_destroy( tempwidget );
    return ;
};

gchar *b2s( gboolean b )
{
    if ( b )
        return ( "TRUE" );
    else
        return ( "FALSE" );
};

gboolean s2b( gchar *tmp )
{
    g_assert( tmp != NULL );
    if ( g_ascii_strncasecmp( "true", tmp, 4 ) == 0 )
        return ( TRUE );
    else
        return ( FALSE );
};

user_info *user_info_init( void )
{
    /* contructor for user_info, and other poop. */
    user_info * temp;

    temp = ( user_info * ) g_malloc0 ( sizeof( user_info ) );
    if ( temp == NULL )
        g_error( "Failed to allocate memory for new user_info structure\n" );
    temp->myname = NULL;
    temp->defaultalias = NULL;
    temp->contacts = NULL;
    temp->groups = NULL;
    temp->myid = NULL;
    temp->file_permissions = USER_ALLOW_CHECK;
    temp->key_passphrase = NULL;
    temp->keylength = 0;
    temp->localport = CUT_DEFAULT_PORT;

    temp->cutlass_info = cutlass_init();
    if ( temp->cutlass_info == NULL )
        g_error ( "couldn't initialize cutlass network/crypto subsystem.  croaking. \n" );

    temp->newuser = TRUE;
    temp->verbosity = CUT_QUIET;

    return ( temp );
};

contact_info *contact_info_init( void )
{
    /* a convience contructor function for contact_info */
    contact_info * newcontact;

    newcontact = ( contact_info * ) g_malloc0( sizeof( contact_info ) );
    if ( newcontact == NULL )
        g_error( "failed to allocate memory for new contact.\n" );
    newcontact->contactname = NULL;
    newcontact->contacthostname = NULL;
    newcontact->nickname = NULL;
    newcontact->identification = NULL;
    newcontact->icon = NULL;
    newcontact->connected = FALSE;
    newcontact->firstcontact = 0;
    newcontact->lastcontact = 0;
    newcontact->textcapable = 1;		//assume *this* is possible;
    newcontact->audiocapable = 0;
    newcontact->videocapable = 0;
    newcontact->fileserver = 0;
    newcontact->port = CUT_DEFAULT_PORT;

    return ( newcontact );
};

contact_group *contact_group_init( void )
{
    contact_group * temp;

    temp = g_malloc0( sizeof( contact_group ) );
    temp->groupname = NULL;
    temp->uuid = NULL;
    temp->contacts = NULL;
    temp->icon = NULL;

    return ( temp );
}

gint contact_group_destroy( contact_group *cg )
{
    contact_info * ci = NULL;

    if ( cg == NULL )
        return ( 0 );

    if ( cg->groupname != NULL )
        g_free( cg->groupname );
    if ( cg->uuid != NULL )
        g_free( cg->uuid );
    if ( cg->icon != NULL )
        g_free( cg->icon );

    // need to free the contacts and groups CList
    if ( cg->contacts != NULL )
    {
        ci = ( contact_info * ) g_slist_next( cg->contacts );
        while ( ci != NULL )
        {
            contact_info_destroy( ci );
            ci = ( contact_info * ) g_slist_next( cg->contacts );
        }
    }
    g_free ( cg );
    cg = NULL;
    return ( 0 );
}

gint user_info_destroy ( user_info *user_stuff )
{
    GSList * tmplist = NULL;

    if ( user_stuff == NULL )
        return ( 0 );

    if ( user_stuff->myname != NULL )
        g_free ( user_stuff->myname );
    if ( user_stuff->defaultalias != NULL )
        g_free ( user_stuff->defaultalias );
    if ( user_stuff->myid != NULL )
        g_free( user_stuff->myid );

    // need to free the contacts and groups CList
    if ( user_stuff->contacts != NULL )
    {

        while ( user_stuff->contacts != NULL )
        {
            if ( user_stuff->contacts->next != NULL )
                tmplist = user_stuff->contacts->next;
            if ( user_stuff->contacts->data != NULL )
                contact_info_destroy( ( contact_info * ) user_stuff->contacts->data );
            else
                tmplist = NULL;		// will force the end of the loop, 'cause
            //	of the line below.
            user_stuff->contacts = tmplist;
        }

    }
    if ( user_stuff->groups != NULL )
    {
        while ( user_stuff->groups != NULL )
        {
            tmplist = user_stuff->groups->next;
            if ( user_stuff->groups->data != NULL )
                contact_group_destroy( user_stuff->groups->data );
            else
                tmplist = NULL;		// will force the end of the loop, 'cause
            //	of the line below.
            user_stuff->groups = tmplist;
        }
    }

    free_private_key( user_stuff->key );

    if ( user_stuff->key_passphrase != NULL )
        memset ( user_stuff->key_passphrase, 1, g_utf8_strlen( user_stuff->key_passphrase, 0 ) );

    printf ( "cutlass_shutdown_all:" );
    if ( user_stuff->cutlass_info != NULL )
    {
        cutlass_shutdown_all( user_stuff->cutlass_info );
        g_print ( " done\n" );
    }
    else
        g_print ( " unneeded\n" );

    g_free( user_stuff );
    user_stuff = NULL;
    return ( 0 );
};

void *contact_info_destroy( gpointer data )
{
    contact_info * ci;

    ci = ( contact_info * ) data;
    if ( ci == NULL )
        return ( NULL );
    if ( ci->contactname != NULL )
        g_free( ci->contactname );
    if ( ci->contacthostname != NULL )
        g_free( ci->contacthostname );
    if ( ci->nickname != NULL )
        g_free( ci->nickname );
    if ( ci->identification != NULL )
        g_free( ci->identification );
    if ( ci->icon != NULL )
        g_free( ci->icon );

    g_free( ci );
    ci = NULL;
    return ( NULL );
};

gint validate_contactinfo( contact_info *ci )
/* really simple validation.  return the appropriate GTKC_ERR_* for whatever is
 * wrong.
 */
{
    if ( ci == NULL )
        return ( GTKC_ERR_NULL );

    if ( ci->contactname == NULL )
        return ( GTKC_ERR_NONAME );
    if ( strlen ( ci->contactname ) <= 0 )
        return ( GTKC_ERR_NONAME );
    if ( ci->textcapable == FALSE )
        return ( GTKC_ERR_NOTEXT );
    /* not sure how this next one would get hit, since the above would
     * catch the !textcapable 
     */
    if ( ! ( ci->textcapable || ci->audiocapable || ci->videocapable || ci->videocapable ) )
        return ( GTKC_ERR_NOCAP );

    return ( 0 );
};

gchar *get_user_config_file_name( void )
/* this may not work with NIS/LDAP/NIS+, etc. not tested, either. */
{
    gchar * directory;
    gchar *tmp;

    tmp = g_strdup( g_get_home_dir() );
    directory = g_strconcat ( tmp, "/.gtkcutlass/userinfo", NULL );

    g_free ( tmp );
    return ( directory );
};

gchar *get_contacts_config_file_name( void )
/* this may not work with NIS/LDAP/NIS+, etc. not tested, either. */
{
    gchar * directory;
    gchar *tmp;

    tmp = g_strdup( g_get_home_dir() );
    directory = g_strconcat ( tmp, "/.gtkcutlass/contacts", NULL );

    g_free ( tmp );
    return ( directory );
};

gchar *get_groups_config_file_name( void )
/* this may not work with NIS/LDAP/NIS+, etc. not tested, either. */
{
    gchar * directory;
    gchar *tmp;

    tmp = g_strdup( g_get_home_dir() );
    directory = g_strconcat ( tmp, "/.gtkcutlass/groups", NULL );

    g_free ( tmp );
    return ( directory );
};

gchar *get_key_save_file_name( void )
{
    gchar * directory;
    gchar *tmp;

    tmp = g_strdup( g_get_home_dir() );
    directory = g_strconcat ( tmp, "/.gtkcutlass/userkey", NULL );
    return ( directory );
}

gint save_user_key( void )
{
    extern user_info * userstuff;
    FILE *savefile;

    savefile = fopen( get_key_save_file_name(), "w" );
    if ( savefile == NULL )
    {
        g_warning ( " save key failed: %s\n", g_strerror( errno ) );
        return ( -1 );
    }

    if ( userstuff->key_passphrase == NULL )
    {
        send_statusbar_message( "what", "No saved passphrase. Not re-saving key." );
        return ( 0 );
    }
    // need checks for validity/presence of key, passphrase stuff.
    save_private_key( userstuff->key, userstuff->key_passphrase, savefile );
    if ( chmod ( get_key_save_file_name(), S_IRUSR | S_IWUSR ) != 0 )
        g_error ( "something screwey happened saving the key.  Aborting. \n" );
    fclose ( savefile );
    return ( 0 );
}


gint save_user_info( void )
/* OK.  must be careful here.  I am going to save to $HOME/.gtkcutlass/userinfo
 * but I have to be sure that everything is resonably safe. <sigh> Damn 
 * security.
 */

/* some of the user_info stuff isn't stored.  like icon, for example. */
/* Also, the loading may get b0rked if there are \n's and stuff in the
 * username, hostname, etc fields. 
 */

/* some of the user stuff isn't stored, but needs to be.  the freekin' DH key, for example.  */
{
    gchar * configfilename = NULL;
    gchar *configdir = NULL;
    GString *configdata;
    FILE *configfile = NULL;
    guint written = 0;
    extern user_info *userstuff;

    configfilename = get_user_config_file_name();
    if ( configfilename == NULL )
        g_error( "Cannot determine home dir.  exiting\n" );

    configdir = ( gchar * ) dirname( get_user_config_file_name() );

    if ( ! g_file_test ( configdir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR ) )
        if ( mkdir ( configdir, 0700 ) != 0 )
            g_error( "couldn't create %s.  Aborting\n", configdir );

    configfile = fopen ( configfilename, "w" );
    if ( configfile == NULL )        		// this will dump core.  FIXME: Be careful. may want to zero mem first.
        g_error ( "couldn't open save file - %s.\n  The error was :%s\n", configfilename, g_strerror( errno ) );

    /* add a header */
    configdata = g_string_new( "[GTK-Cutlass schema -1- save file]\n" );

    /* user-specific info */
    g_string_append_printf( configdata, "[myname] %s\n", userstuff->myname );
    g_string_append_printf( configdata, "[defaultalias] %s\n", userstuff->defaultalias );
    g_string_append_printf( configdata, "[myid] %s\n", userstuff->myid );
    g_string_append_printf( configdata, "[audiocapable] %s\n", b2s( userstuff->audiocapable ) );
    g_string_append_printf( configdata, "[textcapable] %s\n", b2s( userstuff->textcapable ) );
    g_string_append_printf( configdata, "[videocapable] %s\n", b2s( userstuff->videocapable ) );
    g_string_append_printf( configdata, "[fileserver] %s\n", b2s( userstuff->fileserver ) );
    g_string_append_printf( configdata, "[localport] %u\n", userstuff->localport );
    g_string_append_printf( configdata, "[verbosity] %d\n", userstuff->verbosity );
    g_string_append_printf( configdata, "[fileperms] %d\n", userstuff->file_permissions );

    written = g_fprintf( configfile, "%s", configdata->str );
    if ( written != strlen ( configdata->str ) )
    {
        g_warning( "didn't complete the write.  only wrote %u of %u characters (bytes). \n", written, configdata->len );
        g_free ( configfilename );
        memset( configdata->str, 0, configdata->len );
        g_string_free( configdata, TRUE );
        fclose ( configfile );		// didn't check return, I know.
        return ( GTKC_INCOMPLETE_WRITE );
    }


    /* the data structure itself is sorta sensitive.  I am going to zero it before freeing it */
    memset( configdata->str, 0, configdata->len );
    g_string_free( configdata, TRUE );
    g_free ( configdir );
    g_free ( configfilename );
    fclose ( configfile );		// didn't check return, I know.
    return ( 0 );
}

gint save_contact_list( void )
{
    GString * configdata = NULL;
    GSList *foocounter = NULL;
    contact_info *foo = NULL;
    gchar *configfilename = NULL;
    gchar *configdir = NULL;
    FILE *configfile = NULL;
    guint written = 0;
    extern user_info *userstuff;

    configdata = g_string_new( "[GTK-Cutlass schema -1- save file]\n" );

    /* contacts */
    g_string_append_printf( configdata, "[contacts] --\n" );

    foocounter = ( GSList * ) userstuff->contacts;
    if ( foocounter == NULL )         							// perfectly valid to
        //	have no contacts.  Lonely, but possible.
    {
        g_string_free( configdata, TRUE );
        return ( 0 );
    }
    foo = ( contact_info * ) foocounter->data;
    while ( foo != NULL )
    {
        g_string_append_printf( configdata, "[username] %s\n", foo->contactname );
        g_string_append_printf( configdata, "[nickname] %s\n", foo->nickname );
        g_string_append_printf( configdata, "[hostname] %s\n", foo->contacthostname );
        g_string_append_printf( configdata, "[lastcontact] %ld\n", foo->lastcontact );
        g_string_append_printf( configdata, "[firstcontact] %ld\n", foo->lastcontact );
        g_string_append_printf( configdata, "[textcapable] %s\n", b2s( foo->textcapable ) );
        g_string_append_printf( configdata, "[audiocapable] %s\n", b2s( foo->audiocapable ) );
        g_string_append_printf( configdata, "[videocapable] %s\n", b2s( foo->videocapable ) );
        g_string_append_printf( configdata, "[hostname] %s\n", foo->contacthostname );
        g_string_append_printf( configdata, "[fileserver] %s\n", b2s( foo->fileserver ) );
        g_string_append_printf( configdata, "[identification] %s\n", foo->identification );
        g_string_append_printf( configdata, "[port] %u\n", foo->port );
        foocounter = foocounter->next;
        if ( foocounter == NULL )
            foo = NULL;
        else
            foo = ( contact_info * ) foocounter->data;
    }

    configfilename = get_contacts_config_file_name();
    if ( configfilename == NULL )
        g_error( "Cannot determine home dir.  exiting\n" );

    configdir = ( gchar * ) dirname( get_contacts_config_file_name() );

    if ( ! g_file_test ( configdir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR ) )
        if ( mkdir ( configdir, 0700 ) != 0 )
            g_error( "couldn't create %s.  Aborting\n", configdir );

    configfile = fopen ( configfilename, "w" );
    if ( configfile == NULL )        		// this will dump core.  FIXME: Be careful. may want to zero mem first.
        g_error ( "couldn't open contact save file - %s.\n  The error was :%s\n", configfilename, g_strerror( errno ) );

    written = g_fprintf( configfile, "%s", configdata->str );
	g_free ( configfilename );
    memset( configdata->str, 0, configdata->len );
    g_string_free( configdata, TRUE );
    fclose ( configfile );		// didn't check return, I know.
    if ( written != configdata->len )
    {
        g_warning( "didn't complete the write.  only wrote %u of %u characters (bytes). \n", written, configdata->len );
        return ( GTKC_INCOMPLETE_WRITE );
    }
    return ( 0 );
}

gui_status *gui_status_init( void )
{
    gui_status * temp;
    guint starttime = 0;

    starttime = time( NULL );
    temp = g_malloc0( sizeof( gui_status ) );
    temp->XMLstuff = NULL;
    temp->selection_is_contacts = -1;
    temp->newdude = FALSE;
    temp->something_selected = FALSE;
    temp->Tree = NULL;
    temp->selectedname = NULL;
    temp->MainWinXML = NULL;
    temp->audio_threshhold = 1000;
    temp->unsavedchanges = FALSE;
    temp->activeconversation = FALSE;
    return ( temp );
}

conv_conn *conv_conn_init ( void )
{
    conv_conn * tmp;

    tmp = ( conv_conn * ) g_malloc0( sizeof ( conv_conn ) );

    // the connection structure will be zeroed by the g_malloc0, above.
    tmp->hostname = NULL;
    tmp->name = NULL;
    tmp->port = CUT_DEFAULT_PORT;
    tmp->addr = NULL;
    tmp->textview = NULL;
    tmp->entry = NULL;
    tmp->id = -1;
    tmp->winder = NULL;
    tmp->was_connected = 1;
    tmp->is_connected = 1;
    return ( tmp );
}

conv_conn * find_conv_conn_from_toplevel ( GtkWidget *widget )
{
    extern gui_status * currentstatus;
    conv_conn *tmp = NULL;
    GSList *working;

    if ( widget == NULL )
        return ( NULL );

    if ( currentstatus->conversations == NULL )
        return ( NULL );

    working = currentstatus->conversations;

    if ( working->data == NULL )
        return ( NULL );

    tmp = ( conv_conn * ) working->data;

    while ( working != NULL )
    {
        if ( widget == tmp->winder )
            return ( tmp );
        else
            working = working->next;
        if ( working != NULL )
            tmp = ( conv_conn * ) working->data;
    }
    g_print ( "find_conv_conn_from_connection: conn not found\n" );
    return ( NULL );
}



conv_conn * find_conv_conn_from_connection ( uint8_t conn[ CUT_ID_LEN ] )
{
    extern gui_status * currentstatus;
    conv_conn *tmp;
    GSList *working;

    if ( conn == NULL )
        return ( NULL );

    if ( currentstatus->conversations == NULL )
        return ( NULL );

    working = currentstatus->conversations;

    if ( working->data == NULL )
        return ( NULL );

    tmp = ( conv_conn * ) working->data;

    while ( working != NULL )
    {
        if ( memcmp ( tmp->connection, conn, CUT_ID_LEN ) == 0 )
            return ( tmp );
        else
            working = working->next;
        if ( working != NULL )
            tmp = ( conv_conn * ) working->data;
    }
    g_print ( "find_conv_conn_from_connection: conn not found\n" );
    return ( NULL );
}

conv_conn * find_conv_conn_from_textentry ( GtkEntry *entry )
{
    extern gui_status * currentstatus;
    GSList *marker;
    conv_conn *tmp;

    //g_print ( "find_conv_conn_from_textentry: in\n" );
    if ( entry == NULL )
        return ( NULL );
    if ( currentstatus == NULL )
    {
        g_warning ( "find_conv_conn_from_textentry: currentstatus is NULL!\n" );
        return ( NULL );
    }

    if ( currentstatus->conversations == NULL )
        return ( NULL );

    marker = ( GSList * ) currentstatus->conversations;

    if ( marker->data == NULL )
    {
        g_warning ( "No conversations maintained.\n" );
        return ( NULL );
    }

    tmp = ( conv_conn * ) marker->data;
    while ( marker != NULL )
    {
        tmp = ( conv_conn * ) marker->data;
        //	g_print ("fccft: %p:%p\n",tmp->entry,entry);
        if ( tmp->entry == entry )
            return ( tmp );
        else
            marker = marker->next;
    }
    g_print ( "find_conv_conn_from_textentry: textentry not found\n" );
    //g_print ( "find_conv_conn_from_textentry: out\n" );
    return ( NULL );
}

void conv_conn_destroy ( conv_conn *tmp )
{
    extern gui_status * currentstatus;
	GSList *listpointer=NULL;

    if ( tmp == NULL )
        return ;
    if (tmp->name != NULL)
		g_free(tmp->name);
    if (tmp->hostname != NULL)
		g_free(tmp->hostname);
	if ( tmp->hostname != NULL )
        g_free( tmp->hostname );
	if ( tmp->addr != NULL )
        g_free( tmp->addr );
	
	listpointer = g_slist_find (currentstatus->conversations, (gconstpointer) tmp);
	if (listpointer == NULL)
	{
		g_warning ("cannot find entry in conversations list.");
		return;
	}
    currentstatus->conversations = g_slist_remove ( currentstatus->conversations, listpointer);
	g_free( tmp );
}

gint application_crankup( user_info *ui, gui_status *gs )
{
    /*
     * This just bundles alot of the libcutlass start up stuff into a 
     * single function
     */
    struct stat * filestat;
    gint status = 0;
    guint buffersize = 0;
    gint readsize = 0;
    gint fd = 0;
    gint idleid;

    if ( ui == NULL )
        return ( -1 );
    if ( gs == NULL )
        return ( -2 );

    // Set the nick
    if ( ui->defaultalias != NULL )
    {
        if ( cutlass_nick_change( ui->cutlass_info, ui->defaultalias ) != 0 )
        {
            g_warning ( "cutlass could not set default alias as nickname\n" );
            status = -1;
        }
    }
    else
        if ( ui->myname != NULL )
            if ( cutlass_nick_change( ui->cutlass_info, ui->myname ) != 0 )
            {
                g_warning ( "cutlass could not set nick as realname\n" );
                status -= 1;
            }

    if ( cutlass_set_port( ui->cutlass_info, ui->localport ) != 0 )
    {
        g_warning ( "cutlass could not set localport\n" );
        status -= 1;
    }

    if ( ( ui->verbosity <= CUT_DEBUG ) && ( ui->verbosity >= CUT_QUIET ) )
        if ( cutlass_set_verbose( ui->cutlass_info, ui->verbosity ) != 0 )
        {
            g_warning ( "cutlass couldn't set verbosity\n" );
            status -= 1;
        }

    filestat = ( struct stat * ) g_malloc0( sizeof ( struct stat ) );

    if ( stat (GTK_CUTLASS_GLADEDIR "gtkcutlass.glade", filestat ) != 0 )
    {
        status = -2;
        g_free ( filestat );
        return ( status );
    }

    buffersize = filestat->st_size;
    gs->XMLstuff = g_malloc0( sizeof ( gchar ) * buffersize );


    fd = open (GTK_CUTLASS_GLADEDIR "gtkcutlass.glade", O_RDONLY );
    readsize = read ( fd, gs->XMLstuff, buffersize );
    close ( fd );

    if ( readsize != buffersize )
        g_error ( "crazyness with the loading of gtkcutlass.glade" );

    gs->XMLstuffsize = readsize;

    idleid = g_timeout_add ( 250 /* .25 seconds */, parse_gtkc_messages, NULL );
    if ( idleid == 0 )
        g_error ( "couldn't add timeout message parsing function. \n" );

    g_free( filestat );
    return ( status );
}

user_info *load_user_info( void )
{

    extern user_info * userstuff;
    extern gui_status *currentstatus;
    guint filesize = 0;
    gint newuser = 0;
    gchar *configfilename = NULL;
    gchar *configdir = NULL;
    gchar *buf = NULL;
    gchar **elements;
    gint response = 0;
    gchar *tmp;
    gint x = 0;
    struct stat *filestat = NULL;
    gint configfile;
    gint meh = 0;

    send_statusbar_message( "userinfo", "Loading  ..." );

    configfilename = get_user_config_file_name();
    configdir = dirname( get_user_config_file_name() );

    if ( configfilename == NULL )
        g_error( "couldn't determine home directory or user config filename\n" );
    if ( configdir == NULL )
        g_error( "couldn't determine home directory or user config directory\n" );

    if ( ! g_file_test ( configdir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR ) )
        newuser = 1;
    if ( ! g_file_test ( configfilename, G_FILE_TEST_EXISTS ) )
        newuser = 1;

    if ( newuser == 1 )
    {
        currentstatus->newdude = TRUE;
        g_free( configfilename );
        g_free( configdir );
        run_user_info_panel();
        return ( userstuff );
    }

    filestat = ( struct stat * ) g_malloc0( sizeof ( struct stat ) );

    if ( stat ( configfilename, filestat ) != 0 )
    {
        tmp = g_strdup_printf( "Error Loading %s", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ( NULL );
    }

    filesize = ( guint ) filestat->st_size;

    g_free( filestat );

    if ( filesize >= ( 1024 * 1024 ) )
    {
        tmp = g_strdup_printf( "User information database (%s) corrupted. (Size > 1M)\n", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ( NULL );
    }

    buf = ( gchar * ) g_malloc0( filesize );

    configfile = open ( configfilename, O_RDONLY );
    if ( configfile <= 2 )
    {
        tmp = g_strdup_printf( "Error opening file %s.\n", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ( NULL );
    }

    if ( read( configfile, buf, filesize ) != filesize )
    {
        tmp = g_strdup_printf( "User configuration file changed as I read it.  (%s)\n", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ( NULL );
    }
    // OK.  At this point, buf should hold the contents of the config file;
    // ready for parsing. if newuser is set to (not 0) then run the info panel.

    // split into an array of char *'s
    elements = g_strsplit( buf, "\n", 2048 );

    x = 0;
    while ( elements[ x ] != NULL )
    {
        if ( g_ascii_strncasecmp( "[myname] ", elements[ x ], 9 ) == 0 )
            userstuff->myname = g_strdup( elements[ x ] + 9 );
        if ( g_ascii_strncasecmp( "[defaultalias] ", elements[ x ], 15 ) == 0 )
            userstuff->defaultalias = g_strdup( elements[ x ] + 15 );
        if ( g_ascii_strncasecmp( "[audiocapable] ", elements[ x ], 15 ) == 0 )
            userstuff->audiocapable = s2b( elements[ x ] + 15 );
        if ( g_ascii_strncasecmp( "[textcapable] ", elements[ x ], 14 ) == 0 )
            userstuff->textcapable = s2b( elements[ x ] + 14 );
        if ( g_ascii_strncasecmp( "[videocapable] ", elements[ x ], 15 ) == 0 )
            userstuff->videocapable = s2b( elements[ x ] + 15 );
        if ( g_ascii_strncasecmp( "[fileserver] ", elements[ x ], 13 ) == 0 )
            userstuff->fileserver = s2b( elements[ x ] + 13 );
        if ( g_ascii_strncasecmp( "[fileperms] ", elements[ x ], 11 ) == 0 )
            userstuff->file_permissions = ( gint ) atoi( elements[ x ] + 11 );
        if ( g_ascii_strncasecmp( "[verbosity] ", elements[ x ], 11 ) == 0 )
            userstuff->verbosity = ( gint ) atoi( elements[ x ] + 11 );
        if ( g_ascii_strncasecmp( "[myid] ", elements[ x ], 7 ) == 0 )
            userstuff->myid = g_strdup( elements[ x ] + 7 );
        if ( g_ascii_strncasecmp( "[localport] ", elements[ x ], 11 ) == 0 )
            userstuff->localport = ( guint ) strtoul( elements[ x ] + 11, NULL, 10 );
        x++;
    }

    if ( userstuff->textcapable )
        cutlass_set_permission ( userstuff->cutlass_info, CAN_RECV_MSGS, ALL_ALLOWED );
    if ( userstuff->videocapable )
        cutlass_set_permission ( userstuff->cutlass_info, CAN_RECV_VIDEO, USER_ALLOW_CHECK );
    if ( userstuff->audiocapable )
        cutlass_set_permission ( userstuff->cutlass_info, CAN_RECV_AUDIO, USER_ALLOW_CHECK );
    if ( userstuff->fileserver )
        cutlass_set_permission ( userstuff->cutlass_info, CAN_RECV_FILES, userstuff->file_permissions );

    if ( userstuff->defaultalias != NULL )
    {
        if ( cutlass_nick_change( userstuff->cutlass_info, userstuff->defaultalias ) != 0 )
            g_warning ( "error setting nickname" );
    }
    else
        if ( userstuff->myname != NULL )
        {
            if ( cutlass_nick_change( userstuff->cutlass_info, userstuff->myname ) != 0 )
                g_warning ( "error setting nickname" );
        }
        else
            g_warning ( "couldn't set nick (nothing worked) - <sniff>" );


    g_free( buf );
    g_strfreev( elements );
    close( configfile );
    g_free( configfilename );

    load_contact_info();
    load_groups_info();

    if ( get_user_key () != 0 )
    {
        g_free ( userstuff->key_passphrase );
        userstuff->key_passphrase = NULL;
        response = YesNo ( NULL, "Key loading failed.  Retry?" );
        while ( ( response == GTK_RESPONSE_YES ) && ( get_user_key() != 0 ) )
        {
            g_free ( userstuff->key_passphrase );
            userstuff->key_passphrase = NULL;
            response = YesNo ( NULL, "Key loading failed.  Retry?" );
        }
        if ( response == GTK_RESPONSE_NO )
        {
            user_info_destroy ( userstuff );
            gtk_main_quit();
        }
    }

    meh = check_user_key();

    if ( meh == -1 )
        g_error ( "userinfo is NULL!" );
    if ( meh == -2 )
        g_error ( "Passphrase is NULL!\n" );
    /*	if (meh == -3)
    		g_error ("zero-length key loaded!\n"); 
    */
    if ( meh == -4 )
        g_error ( "NULL key loaded!\n" );

    if ( cutlass_set_private_key ( userstuff->cutlass_info, userstuff->key ) != 0 )
        g_error ( "Error setting user private key\n" );

    // Baaaaaaad to do the following - oh frickin' well.
    if ( userstuff->cutlass_info->local_key.key == NULL )
        g_error ( "key is fucked up.\n" );


    send_statusbar_message( "userinfo", g_strdup_printf( "Loaded :%s", userstuff->myname ) );

    g_free( configdir );
    return ( userstuff );
}


void load_contact_info( void )
/* this will load the user's contact info into userinfo->contacts (its a gslist) */
{
    extern user_info * userstuff;
    contact_info *newcontact;
    gint notfirst = 1;
    guint filesize = 0;
    gchar *configfilename = NULL;
    gchar *configdir = NULL;
    gchar *buf = NULL;
    gchar **elements;
    gchar *tmp;
    gint x = 0;
    struct stat *filestat = NULL;
    gint configfile;


    send_statusbar_message( "contacts", "Loading (contacts) ..." );

    configfilename = get_contacts_config_file_name();
    configdir = dirname( get_contacts_config_file_name() );

    if ( configfilename == NULL )
        g_error( "couldn't determine home directory for contact config filename\n" );
    if ( configdir == NULL )
        g_error( "couldn't determine home directory or contact config directory\n" );

    if ( ! g_file_test ( configfilename, G_FILE_TEST_EXISTS ) )
        return ;

    filestat = ( struct stat * ) g_malloc0( sizeof ( struct stat ) );

    if ( stat ( configfilename, filestat ) != 0 )
    {
        tmp = g_strdup_printf( "Error Loading %s", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ;
    }

    filesize = ( guint ) filestat->st_size;

    g_free( filestat );

    if ( filesize >= ( ( 1024 * 1024 ) * 10 ) )
    {
        tmp = g_strdup_printf( "Contact information database (%s) corrupted.  (Size > 10M)\n", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ;
    }

    buf = ( gchar * ) g_malloc0( sizeof ( gchar ) * filesize );

    configfile = open ( configfilename, O_RDONLY );
    if ( configfile == -1 )
    {
        send_statusbar_message( "whoops", "No contact information" );
        return ;
    }
    if ( configfile <= 2 )
    {
        tmp = g_strdup_printf( "Error opening file %s.\n", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ;
    }

    if ( read( configfile, buf, filesize ) != filesize )
    {
        tmp = g_strdup_printf( "User configuration file changed as I read it.  (%s)\n", configfilename );
        tellem( tmp );
        g_free( tmp );
        return ;
    }
    // OK.  At this point, buf should hold the contents of the config file;
    // ready for parsing. if newuser is set to (not 0) then run the info panel.

    // split into an array of char *'s

    elements = g_strsplit( buf, "\n", 2048 );

    if ( elements == NULL )
    {
        tmp = g_strdup_printf( "Could not parse %s", configfilename );
        send_statusbar_message( "hmmmm", tmp );
        g_free( tmp );
        return ;
    }

    if ( g_ascii_strncasecmp( "[GTK-Cutlass schema", elements[ 0 ], 19 ) != 0 )
    {
        tmp = g_strdup_printf( "Could not parse %s", configfilename );
        send_statusbar_message( "hmmmm", tmp );
        g_free( tmp );
        return ;
    }

    if ( elements[ 0 ][ 21 ] != '1' )
        g_error ( "%s is the incrorrect schema (I should handle this better)\n", configfilename );


    newcontact = contact_info_init();

    notfirst = 1;
    x = 2;


    while ( elements[ x ] != NULL )
    {
        if ( g_ascii_strncasecmp( "[username] ", elements[ x ], 11 ) == 0 )
        {
            if ( notfirst == 0 )
            {
                userstuff->contacts = g_slist_append ( userstuff->contacts, newcontact );
                append_treestore_contactinfo( newcontact );
                newcontact = contact_info_init();
                newcontact->contactname = g_strdup( elements[ x ] + 11 );
            }
            else
            {
                newcontact->contactname = g_strdup( elements[ x ] + 11 );
                notfirst = 0;
            }
        }
        if ( g_ascii_strncasecmp( "[nickname] ", elements[ x ], 11 ) == 0 )
            newcontact->nickname = g_strdup( elements[ x ] + 11 );
        if ( g_ascii_strncasecmp( "[lastcontact] ", elements[ x ], 13 ) == 0 )
            newcontact->lastcontact = ( guint ) strtoul ( elements[ x ] + 13, NULL, 10 );
        if ( g_ascii_strncasecmp( "[firstcontact] ", elements[ x ], 14 ) == 0 )
            newcontact->firstcontact = ( guint ) strtoul ( elements[ x ] + 14, NULL, 10 );
        if ( g_ascii_strncasecmp( "[hostname] ", elements[ x ], 11 ) == 0 )
            newcontact->contacthostname = g_strchug(g_strndup( ( elements[ x ] + 11 ), 254 ));
        if ( g_ascii_strncasecmp( "[textcapable] ", elements[ x ], 13 ) == 0 )
            newcontact->textcapable = s2b( elements[ x ] + 13 );
        if ( g_ascii_strncasecmp( "[audiocapable] ", elements[ x ], 14 ) == 0 )
            newcontact->audiocapable = s2b( elements[ x ] + 14 );
        if ( g_ascii_strncasecmp( "[videocapable] ", elements[ x ], 14 ) == 0 )
            newcontact->videocapable = s2b( elements[ x ] + 14 );
        if ( g_ascii_strncasecmp( "[fileserver] ", elements[ x ], 10 ) == 0 )
            newcontact->fileserver = s2b( elements[ x ] + 10 );
        if ( g_ascii_strncasecmp( "[identification] ", elements[ x ], 16 ) == 0 )
            newcontact->identification = g_strdup( elements[ x ] + 16 );
        if ( g_ascii_strncasecmp( "[port] ", elements[ x ], 6 ) == 0 )
            newcontact->port = ( guint ) strtoul( elements[ x ] + 6, NULL, 10 );
        x++;
    }
    userstuff->contacts = g_slist_append ( userstuff->contacts, newcontact );
    append_treestore_contactinfo( newcontact );

    send_statusbar_message( "userinfo", "Loaded" );
    g_free( buf );
    g_strfreev( elements );
    close( configfile );
    g_free( configfilename );
    g_free( configdir );
    return ;

}
void load_groups_info( void )
{
    /*
    	extern user_info *userstuff;
    	gint newuser=0;
    	guint filesize=0;
    	gchar *configfilename = NULL;
    	gchar *configdir = NULL;
    	gchar *buf=NULL;	
    	gchar **elements;
    	gchar *tmp;
    	gint x=0;
    	struct stat *filestat=NULL;
    	gint configfile;
    	
     
    	send_statusbar_message("userinfo","Loading ...");
     
    	configfilename = get_user_config_file_name();
    	configdir = dirname(get_user_config_file_name());
     
    	if (configfilename == NULL)
    		g_error("couldn't determine home directory or user config filename\n");
    	if (configdir == NULL)
    		g_error("couldn't determine home directory or user config directory\n");
     
    	if (! g_file_test (configdir,G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
    	{
    		newuser=1;
    		g_print ("no dir\n");
    	}
    	if (! g_file_test (configfilename,G_FILE_TEST_EXISTS ))
    	{
    		newuser=1;
    		g_print ("no file\n");
    	}
    	
    	if ( newuser == 1)
    	{
    		g_free(configfilename);
    		g_free(configdir);
    		return (run_user_info_panel(userstuff));
    	}
     
    	filestat = (struct stat *) g_malloc0(sizeof (struct stat));
     
    	if (stat (configfilename, filestat) != 0)
    	{
    		tmp = g_strdup_printf("Error Loading %s",configfilename);
    		tellem(tmp);
    		g_free(tmp);
    		return (NULL);
    	}
    	
    	filesize = (guint) filestat->st_size;
     
    	g_free(filestat);
     
    	if (filesize >= (1024*1024) )
    	{
    		tmp = g_strdup_printf("User information database (%s) corrupted. (Size > 1M)\n",configfilename);
    		tellem(tmp);
    		g_free(tmp);
    		return (NULL);
    	}
     
    	buf = (gchar *) g_malloc0(filesize);	
     
    	configfile = open (configfilename,O_RDONLY);
     
    	if (configfile <= 2 )
    	{
    		tmp = g_strdup_printf("Error opening file %s.\n",configfilename);
    		tellem(tmp);
    		g_free(tmp);
    		return (NULL);
    	}
     
    	if (read(configfile,buf,filesize) != filesize)
    	{
    		tmp = g_strdup_printf("User configuration file changed as I read it.  (%s)\n",configfilename);
    		tellem(tmp);
    		g_free(tmp);
    		return (NULL);
    	}
    	// OK.  At this point, buf should hold the contents of the config file;
    	// ready for parsing. if newuser is set to (not 0) then run the info panel.
     
    	// split into an array of char *'s
    	elements = g_strsplit(buf,"\n",2048);
     
    	x=0;
    	while (elements[x] != NULL )
    	{
    		if (g_ascii_strncasecmp("[myname] ",elements[x],9) == 0)
    			userstuff->myname = g_strdup(elements[x]+9);
    		if (g_ascii_strncasecmp("[defaultalias] ",elements[x],15) == 0)
    			userstuff->defaultalias = g_strdup(elements[x]+15);
    		if (g_ascii_strncasecmp("[myid] ",elements[x],7) == 0)
    			userstuff->myid = g_strdup(elements[x]+7);
    		if (g_ascii_strncasecmp("[localport] ",elements[x],11) == 0)
    			userstuff->myid = g_strdup(elements[x]+11);
    		x++;
    	}
    	send_statusbar_message("userinfo","Loaded");
    	g_free(buf);
    	g_strfreev(elements);
    	close(configfile);
    	g_free(configfilename);
     
    	g_free(configdir);
    	return (userstuff);
    	*/
}

gint check_user_key ( void )
/* Gak.  I need some way of checking the key itself. */
{
    extern user_info * userstuff;
    if ( userstuff == NULL )
        return ( -1 );
    if ( userstuff->key_passphrase == NULL )
        return ( -2 );
    if ( userstuff->keylength == 0 )
        return ( -3 );
    if ( userstuff->key.key == NULL )
        return ( -4 );
    return ( 0 );
};


gint passphrase_dialog_wrapper( const gchar *description, char pass[], guint max_size )
{
    extern user_info * userstuff;

    if ( userstuff->key_passphrase == NULL )
        userstuff->key_passphrase = run_passphrase_dialog();
    if ( userstuff->key_passphrase != NULL )
        strncpy( pass, userstuff->key_passphrase, max_size ); 		// safe?  FIXME: have a clue-infested person check this.
    else
    {
        pass = "";
        return ( 1 );
    }
    return ( 0 );
};

gint get_user_key ( void )
{
    extern user_info * userstuff;
    userstuff->key = load_private_key( get_key_save_file_name(), passphrase_dialog_wrapper );
    if ( userstuff->key.key == NULL )
    {
        free_private_key ( userstuff->key );
        g_free( userstuff->key_passphrase );
        userstuff->key_passphrase = NULL;
        userstuff->key.key = NULL;
        return ( -1 );
    }
    return ( 0 );
};

contact_info *get_contact_from_name ( gchar *name )
{
    extern user_info * userstuff;
    contact_info *ci;
    GSList *marker;

    if ( name == NULL )
        return ( NULL );
	
    marker = userstuff->contacts;
    if ( marker == NULL )
        return ( NULL );


    while ( marker != NULL )
    {
        ci = ( contact_info * ) marker->data;
        if ( ci == NULL )
        {
            g_warning ( "null contact info entry" );
            return ( NULL );
        }
        if ( g_ascii_strcasecmp ( ci->nickname, name ) == 0 )
            return ( ci );
        else
            if ( g_ascii_strcasecmp ( ci->contactname, name ) == 0 )
              	return ( ci );
            else
                marker = marker->next;
    }
    g_warning ( "selected contact not found.  *shouldn't happen!*" );
    return ( NULL );
}

conv_conn *get_conversation_from_conn ( uint8_t conn[ CUT_ID_LEN ] )
{
    extern gui_status * currentstatus;
    conv_conn *cc;
    GSList *marker;


    if ( conn == NULL )
        return ( NULL );

    marker = currentstatus->conversations;
    if ( marker == NULL )
        return ( NULL );

    while ( marker != NULL )
    {
        cc = ( conv_conn * ) marker->data;
        if ( cc == NULL )
            return ( NULL );
        if ( cc->connection[ 0 ] == 0 )
        {
            g_print ( "get_conversation_from_conn: Empty list\n" );
            return ( NULL );
        }
        if ( memcmp ( cc->connection, conn, CUT_ID_LEN ) == 0 )
            return ( cc );
        else
            marker = marker->next;
    }
    return ( NULL );
}

conv_conn *get_conversation_from_address ( const gchar *address )
{
    extern gui_status * currentstatus;
    conv_conn *cc;
    GSList *marker;

    if ( address == NULL )
        return ( NULL );

    marker = currentstatus->conversations;
    if ( marker == NULL )
        return ( NULL );

    while ( marker != NULL )
    {
        cc = ( conv_conn * ) marker->data;
        if ( cc == NULL )
        {
            return ( NULL );
        }
        if ( g_ascii_strcasecmp ( cc->addr, address ) == 0 )
            return ( cc );
        else
            marker = marker->next;
    }
    return ( NULL );
}

conv_conn *get_conversation_from_name ( const gchar *name )
{
    extern gui_status * currentstatus;
    conv_conn *cc;
    GSList *marker;

    if ( name == NULL )
        return ( NULL );

    marker = currentstatus->conversations;
    if ( marker == NULL )
        return ( NULL );

    while ( marker != NULL )
    {
        cc = ( conv_conn * ) marker->data;
        if ( cc == NULL )
        {
            return ( NULL );
        }
        if ( g_ascii_strcasecmp ( cc->name, name ) == 0 )
            return ( cc );
        else
            marker = marker->next;
    }
    return ( NULL );
}

gint connect_to_contact ( contact_info *contact )
{
    extern user_info * userstuff;
    struct hostent *remote;
    struct sockaddr_in *contactmachine;
    gchar *tmp;

    if ( contact == NULL )
        return ( -1 );

    // crank up the connection!
    if ( contact->contacthostname == NULL )
    {
        tmp = g_strdup_printf( "Bad / Nonexistant contact hostname: %s", contact->contacthostname );
        tellem( tmp );
        g_free ( tmp );
        return ( -1 );
    }
    tmp = g_strdup_printf( "looking up %s ...", contact->contacthostname );
    send_statusbar_message( "meh", tmp );
    g_free( tmp );
    remote = gethostbyname ( ( const char * ) contact->contacthostname );
    if ( ( remote == NULL ) || ( remote->h_addr == NULL ) )
    {
        tmp = ( gchar * ) hstrerror ( h_errno );
        tellem( tmp );
        g_free ( tmp );
        return ( -2 ) ;
    }

    contactmachine = ( struct sockaddr_in * ) g_malloc0( sizeof ( struct sockaddr_in ) );
    contactmachine->sin_family = AF_INET;
    contactmachine->sin_port = htons( contact->port );
    contactmachine->sin_addr = *( ( struct in_addr * ) remote->h_addr );

    tmp = g_strdup_printf( "Connecting to %s@%s ...", contact->contactname, inet_ntoa( contactmachine->sin_addr ) );
    send_statusbar_message( "meh", tmp );

    if ( cutlass_connect( userstuff->cutlass_info, contactmachine ) != 0 )
    {
        tellem ( g_strdup_printf( "Error connecting to %s", contact->contacthostname ) );
        g_free( tmp );
        return ( -1 ) ;
    }

    g_free( tmp );
    return ( 0 );
}

conv_conn * setup_incoming_conversation ( cutlass_t *handle, struct cut_action_obj *action_object )
{
    conv_conn *cc;
    extern gui_status *currentstatus;
    gchar tmp_array[ INET_ADDRSTRLEN ];

    if ( handle == NULL )
    {
        g_warning( "setup_incoming_conversation: handle is NULL\n" );
        return ( NULL );
    }
    if ( currentstatus == NULL )
    {
        g_warning( "setup_incoming_conversation: currentstatus is NULL\n" );
        return ( NULL );
    }
    if ( action_object == NULL )
    {
        g_warning( "setup_incoming_conversation: action_object is NULL\n" );
        return ( NULL );
    }

    cc = conv_conn_init();
    cc->is_connected = 0;

    // the following leaks CUT_ID_LEN bytes.
    memcpy ( cc->connection, cutlass_action_fingerprint ( action_object ), CUT_ID_LEN );
    if ( cc->connection[ 0 ] == 0 )
        g_warning ( " setup_incoming_conversation: null fingerprint\n" );
	
    cc->name = cutlass_action_rnick (action_object);
    if ((cc->name == NULL) || (g_utf8_strlen (cc->name, 128) == 0))
	cc->name = g_strdup ("Unknown");
    //g_print ("nick is [%s]\n",cc->name);
    cc->addr = g_strdup(inet_ntop( AF_INET, cutlass_action_addr (action_object), tmp_array, INET_ADDRSTRLEN));
    // FIXME: The following is broken.
    cc->port = 8356;
//    cc->keyset = cutlass_action_rkey ( action_object );
    cc->name = cutlass_action_rnick ( action_object );
    if ( ( cc->name == NULL ) || ( g_utf8_strlen ( cc->name, 128 ) == 0 ) )
        cc->name = g_strdup ( "Unknown" );
    //g_print ("nick is [%s]\n",cc->name);
    cc->addr = g_strdup( inet_ntop( AF_INET, cutlass_action_addr ( action_object ), tmp_array, INET_ADDRSTRLEN ) );
    // FIXME: The following is broken.
    cc->port = 8356;

    // set up window here.

    currentstatus->conversations = g_slist_append( currentstatus->conversations, cc );

    // should check to see if this connection corrosponds to a user we know, and update their status on the mainwin.
    return ( cc );
}

gint wait_for_connection( conv_conn *cc, gint timeout, gulong timerstep )
/* this is a really stupid function.  it will wait until timeout
 * until cc->is_connected flags true
 * timeout is in seconds
 * timerstep is in *micro*seconds (1M per sec)
 */
{
    gulong waited = 0;

    if ( cc == NULL )
        return ( -1 );
    if ( timeout < 0 )
        return ( -2 );
    if ( timerstep <= 0 )
        return ( -3 );

    if ( cc->is_connected == 0 )
        return ( 0 );

    while ( usleep ( timerstep ) )
    {
        waited += timerstep;
        if ( waited >= ( timeout * G_USEC_PER_SEC ) )
            return ( -4 );
        if ( cc->is_connected == 0 )
            return ( 0 );
        while ( gtk_events_pending() )
            gtk_main_iteration();
    }
    return ( -99 );
}

gtkc_message_q *message_init( void )
{
    gtkc_message_q * msg;
    msg = g_malloc0( sizeof ( gtkc_message_q ) );
    msg->id = 0;
    msg->read = FALSE;
    msg->action_object = NULL;
    msg->messagetype = UNINITIALIZED;
    return ( msg );
}

gchar *message_get_data ( gtkc_message_q *msg )
{
    if ( msg != NULL )
        return ( msg->data );
    else
        return ( NULL );
}

gtkc_message_q *message_set_data ( gtkc_message_q *msg, gchar *data )
{
    if ( ( msg == NULL ) || ( data == NULL ) )
        return ( NULL );
    if ( msg->data != NULL )
        g_free ( msg->data );
    msg->data = g_strdup ( data );
    return ( msg );
}

gtkc_message_q *message_set_id ( gtkc_message_q *msg, guint id )
{
    if ( ( msg == NULL ) || ( id <= 0 ) )
        return ( NULL );
    msg->id = id;
    return ( msg );
}

gtkc_message_q *message_set_read ( gtkc_message_q *msg )
{
    if ( msg == NULL )
        return ( NULL );
    msg->read = TRUE;
    return ( msg );
}

gboolean message_get_read ( gtkc_message_q *msg )
{
    if ( msg == NULL )
        return ( FALSE );
    return ( msg->read );
}

enum message_type_t message_get_type ( gtkc_message_q *msg )
{
    if ( msg == NULL )
        return ( UNINITIALIZED );
    return ( msg->messagetype );
}

guint message_get_id ( gtkc_message_q *msg )
{
    if ( msg != NULL )
        return ( msg->id );
    else
        return ( -1 );
}

void message_destroy ( gtkc_message_q *msg )
{
    if ( msg == NULL )
        return ;
    if ( msg->data != NULL )
        g_free( msg->data );
    g_free( msg );
    msg = NULL;
}

void handle_incoming_audio_request ( gtkc_message_q *msg )
{
    extern user_info * userstuff;

    if ( msg == NULL )
        return ;
    if ( YesNo ( NULL, "A user has requested audio chat. \n Accept?" ) == GTK_RESPONSE_YES )
    {
        if ( cutlass_audio_accept( userstuff->cutlass_info, msg->action_object ) != 0 )
            g_warning ( "couldn't accept audio connection" );
    }
    else
    {
        if ( cutlass_audio_reject( userstuff->cutlass_info, msg->action_object ) != 0 )
            g_warning ( "couldn't reject audio connection" );
    }
    return ;
}

void handle_incoming_file_send( gtkc_message_q *msg )
{
    extern user_info * userstuff;
    gchar *tmptext = NULL;
    gchar *remotenick = NULL;
    gchar *fname = NULL;
    uint32_t fsize;
    struct in_addr *addr;
    GtkWidget *filedialog = NULL;
    gchar *filename;

    if ( msg == NULL )
        return ;
    remotenick = cutlass_action_rnick ( msg->action_object );
    fname = cutlass_action_fname ( msg->action_object );
    fsize = cutlass_action_fsize( msg->action_object );
    addr = cutlass_action_addr( msg->action_object );

    tmptext = g_strdup_printf( "%s at %s wants to send you a file (%s), which is %u bytes.\n Accept?",
                               remotenick, inet_ntoa( *addr ), fname, fsize );
    if ( YesNo( msg->cc->winder, tmptext ) == GTK_RESPONSE_NO )
    {
        if ( cutlass_file_reject( userstuff->cutlass_info, msg->action_object ) != 0 )
            g_warning ( "coulnd't reject file request" );
    }
    else
    {
        filedialog = gtk_file_chooser_dialog_new ( "Save as", GTK_WINDOW( msg->cc->winder ), GTK_FILE_CHOOSER_ACTION_SAVE,
                     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL );
        gtk_file_chooser_set_current_name ( GTK_FILE_CHOOSER( filedialog ), fname );
        if ( gtk_dialog_run ( GTK_DIALOG( filedialog ) ) == GTK_RESPONSE_CANCEL )
        {
            if ( cutlass_file_reject( userstuff->cutlass_info, msg->action_object ) != 0 )
            {
                g_warning ( "couldn't reject file request(2)" );
                g_free( tmptext );
                g_free( remotenick );
                g_free( fname );
                gtk_widget_destroy ( filedialog );
                return ;
            }
        }
        else
        {
            filename = gtk_file_chooser_get_filename ( GTK_FILE_CHOOSER( filedialog ) );
            if ( cutlass_file_accept( userstuff->cutlass_info, msg->action_object, filename ) != 0 )
            {
                g_warning ( "couldn't tell cutlass about %s", filename );
                g_free( tmptext );
                g_free( remotenick );
                g_free( fname );
                gtk_widget_destroy ( filedialog );
                return ;
            }
        }
    }
    g_free( remotenick );
    g_free( fname );
    g_free( tmptext );
    gtk_widget_destroy ( filedialog );
    return ;
}

gboolean parse_gtkc_messages ( gpointer data )
{
    extern gui_status * currentstatus;
    gtkc_message_q *msg = NULL;
    gchar *foo = NULL;
    GSList *tmp;

    if ( currentstatus->messages == NULL )
        return ( TRUE );

    tmp = currentstatus->messages;
    if ( tmp->data == NULL )
        return ( TRUE );

    msg = ( gtkc_message_q * ) tmp->data;

    if ( ! msg->read )
    {
        switch ( msg->messagetype )
        {
            case NEW_TEXTWINDOW:
                setup_textchat_window ( msg->cc );
                send_statusbar_message( "bah", "Connected" );
                break;
            case NEW_FILEWINDOW:
                handle_incoming_file_send( msg );
                break;
            case NEW_AUDIOWINDOW:
                g_print ( "New Audio window requested \n" );
                break;
            case NEW_VIDEOWINDOW:
                g_print ( "New Video Window requested \n" );
                break;
            case TEXTMESSAGE:
                foo = g_strdup_printf( "%s> %s", msg->cc->name, msg->data );
                if ( append_message_to_textview ( msg->cc->textview, foo ) != 0 )
                {
                    g_warning ( " Error appending message to textview \n" );
                    g_free ( foo );
                    break;
                }
                g_free ( foo );
                break;
            case AUDIOMESSAGE:
                g_print ( "\t Audio Message!\n" );
                break;
            case AUDIOREQUEST:
                g_print ( "\t Audio Request! w00t!\n" );
                handle_incoming_audio_request ( msg );
                break;
            case VIDEOMESSAGE:
                g_print ( "\t Video Message!\n" );
                break;
            case FILEDATA:
                g_print ( "\tfiledata!\n" );
                break;
            default:
                g_warning ( "\t Bustedness! bad message_type in message queue *****\n" );
                return ( FALSE );
                break;
        }
    }

    msg->read = TRUE;

    currentstatus->messages = g_slist_remove ( currentstatus->messages, ( gconstpointer ) msg );
    message_destroy ( msg );

    return ( TRUE );

}

