/*	gtkcutlass.h - the main header file for gtkcutlass
It defines some stuff.

(c) 2004 John Schweitzer
*/

#ifndef GTKCUTLASS
#define GTKCUTLASS

#include <glib.h>
#include <glib/gprintf.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <cutlass.h>	// the biggie from the main project

#include <stdlib.h>
#include <string.h>

#define GTKC_NUM_TREE_COLUMNS 5		// number of columns in the main tree view
#define COL_NAME 0
#define COL_ONLINE_STATUS 1
#define COL_VIDEO 2
#define COL_AUDIO 3
#define COL_FILE 4

#define GTKC_OK 0
#define GTKC_ERR_NONAME 1
#define GTKC_ERR_NOTEXT 2
#define GTKC_ERR_NOCAP  3
#define GTKC_ERR_NODETAILS 4
#define GTKC_ERR_NULL 5

#define GTKC_INCOMPLETE_WRITE 1

#define GTKC_PASS_MAX 128

#define GTKC_READ_BUF 1024

#define GTKC_PATH_MAX 256

#define GTKC_MAX_HOSTNAME_LEN 256

#define GTKC_ABOUTMESSAGE "\nGTK Cutlass v " PACKAGE_VERSION "  \n \
PRERELEASE\n \
\n \
Expect nothing.  It's easier that way.\n \
\n \
\n \
(c) 2005 - John Schweitzer (thanatos@hoodlum.org),\n \
Todd Macdermid (tmacd@synacklabs.net),\n \
 Jack Lloyd (lloyd@randombit.net), \n \
 Nash Foster (nash@solace.net), and \n \
 Kathy Wang (kwang@synacklabs.net)"

struct user_info_struct
{
    gchar *myname;				// my damn name
    gchar *defaultalias;			// default alias (there maybe per-connection aliases too.
    gchar *myid;				// some semi-unique identifier
    GSList *contacts;			// peeps will be in the contacts list and
    // in a list for each element of the groups
    // list
    GSList *groups;
    cutlass_t *cutlass_info;
    cutlass_private_key key;
    gboolean textcapable;
    gboolean audiocapable;
    gboolean videocapable;
    gboolean fileserver;
    gint file_permissions;
    gchar *key_passphrase;
    gint keylength;
    guint	localport;			// port the user prefers to listen on
    gboolean newuser;
    gint	verbosity;
};


typedef struct user_info_struct user_info;

struct contact_info_struct
{
    gchar *contactname;
    gchar *nickname;
    gchar *contacthostname;
    gchar *icon;
    time_t lastcontact;
    time_t firstcontact;
    gboolean connected;
    gboolean textcapable;
    gboolean audiocapable;
    gboolean videocapable;
    gboolean fileserver;
    gchar *identification;
    guint port;

};
typedef struct contact_info_struct contact_info;

struct conversation_connection_t
{
    uint8_t connection[ CUT_ID_LEN ];		// identifier from libcutlass
    GtkTextView *textview;
    GtkWidget *winder;
    GtkEntry *entry;
    gchar *hostname;
    gchar *name;
    gchar *addr;
    guint port;
    gint is_connected;
    gint was_connected;
    gint id;
};
typedef struct conversation_connection_t conv_conn;

enum message_type_t { UNINITIALIZED, NEW_TEXTWINDOW, NEW_FILEWINDOW, NEW_AUDIOWINDOW, NEW_VIDEOWINDOW, TEXTMESSAGE, AUDIOREQUEST, AUDIOMESSAGE, VIDEOMESSAGE, FILEDATA };

struct _gtkc_message_q_t
{
    gchar *data;
    guint id;
    conv_conn *cc;
    enum message_type_t messagetype;
    struct cut_action_obj *action_object;
    gboolean read;
};
typedef struct _gtkc_message_q_t gtkc_message_q;


struct contact_group_struct
{
    gchar *groupname;
    gchar *uuid;
    GSList *contacts;
    gchar *icon;
};
typedef struct contact_group_struct contact_group;


struct gui_status_struct
{
    GtkTreeIter selection;
    gchar *XMLstuff;
    guint XMLstuffsize;
    gint selection_is_contacts;		// group or single-contacts?
    gboolean something_selected;
    GladeXML *MainWinXML;
    GtkTreeView *Tree;			// mainwin tree
    gboolean newdude;
    gchar *selectedname;
    GSList *conversations;
    gboolean activeconversation;
    gboolean unsavedchanges;
    gint audio_threshhold;
    GSList *messages;
    time_t starttime;			// application start time
};
typedef struct gui_status_struct gui_status;

// System
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

// App
#include <gtkcutlass-mainwin-treeview.h>
#include <gtkcutlass-utilities.h>
#include <gtkcutlass-callbacks.h>
#include <gtkcutlass-userinfo.h>
#include <gtkcutlass-main.h>
#include <gtkcutlass-cutlass.h>
#include <gtkcutlass-mainwin-callbacks.h>
#include <gtkcutlass-textconversation.h>
#include <gtkcutlass-keygenwindow.h>
#include <gtkcutlass-newcontact.h>
#include <gtkcutlass-passphrasewindow.h>
#include <gtkcutlass-aboutbox.h>
#include <gtkcutlass-filetransfer.h>
#include <gtkcutlass-popup-callbacks.h>
#include <gtkcutlass-status-panels.h>
#include <gtkcutlass-audiothreshhold.h>
#include <gtkcutlass-editcontact.h>

#endif
